open Ocamlbuild_plugin
open Command
open Ocamlbuild_pack.Ocaml_compiler
open Ocamlbuild_pack.Ocamldep

let link_from_file link modules_file cmX env build =
  let modules_file = env modules_file in
  let contents_list = string_list_of_file modules_file in
  let src_file = Pathname.update_extension "ml" modules_file in

  let modules_dir = Filename.chop_extension modules_file in
  let getdeps modul =
    let fn = Pathname.concat modules_dir ((String.lowercase modul) ^ ".ml") in
    let deps = module_dependencies_of fn in
    List.map (fun (_, d) -> d) deps in
  let deps = List.fold_right (fun m deps -> (getdeps m) @ deps) contents_list contents_list in

  register_module_dependencies src_file deps;
  link contents_list cmX env build

let byte_pack_mlpack = link_from_file byte_pack_modules
let byte_debug_pack_mlpack = link_from_file byte_debug_pack_modules

;;

dispatch begin function
  | Before_rules ->
      rule "ocaml: mlpack & d.cmo* -> d.cmo & cmi + depends"
	~tags:["ocaml"; "debug"; "byte"]
	~prods:["%.d.cmo"]
	~deps:["%.mlpack"; "%.cmi"]
	(byte_debug_pack_mlpack "%.mlpack" "%.d.cmo");
      rule "ocaml: mlpack & cmo* & cmi -> cmo + depends"
	~tags:["ocaml"; "byte"]
	~prod:"%.cmo"
	~deps:["%.mli"; "%.cmi"; "%.mlpack"]
	(byte_pack_mlpack "%.mlpack" "%.cmo")
  | _ -> ()
end
